/* Copyright 2006, freshmowed software.  All rights reserved. */
  
BarChart = function(aWidth, aHeight, chartHeight) 
{ 
    this.width = aWidth;
    this.height = aHeight;
    this.chartHeight = chartHeight;
    this.chartType = "bar";
    this.topBorderWidth = 12;
    this.leftBorderWidth = 0;
    this.rightBorderWidth = 0;
    this.bottomBorderWidth = 5;
    this.units = null;
    this.prefixUnits = false;
    this.dataSet = new Array();
    this.xScale = new Array();
    this.yScale = new Array();
    this.chartConfig = new Array();
    this.needsRefresh = true;
    this.primaryIndex = 0;
    this.currentMinX = null;
    this.currentMaxX = null;
    this.currentNumBuckets = 1;

    this.countaboutStyle = new CountaboutStyle();
    this.redStart = this.countaboutStyle.redStart;
    this.redEnd = this.countaboutStyle.redEnd;
    this.greenStart = this.countaboutStyle.greenStart;
    this.greenEnd = this.countaboutStyle.greenEnd;
    this.blueStart = this.countaboutStyle.blueStart;
    this.blueEnd = this.countaboutStyle.blueEnd;
}

BarChart.prototype =
{    
    setWidth: function(aWidth)
    {
        var oldWidth = this.width;
        this.width = aWidth;
        var canvas = document.getElementById("barChartCanvas");
        canvas.setAttribute('width', aWidth);  // absolutely critical for leopard
        //var overlayDiv= document.getElementById("canvasAndOverlay");
        //overlayDiv.style.width = this.width + 'px';
    },
    
    setHeight: function(aHeight)
    {
        this.height = aHeight;
        return this;
    },
    
    setChartConfig: function(aChartConfig, anIndex)
    {
       if (anIndex == null) anIndex = 0;
       this.chartConfig[anIndex] = aChartConfig;
       this.needsRefresh = true;
    },
            
    setUnitsString: function(aString, shouldPrefix)
    {
      this.units = aString;
      this.prefixUnits = shouldPrefix;
    },
    
    setData: function(someData, anIndex)
    {
        if ( !someData ) 
        {
            this.dataSet[anIndex] = new Array();
            return;
        }
        
        this.dataSet[anIndex] = someData;
        this.needsRefresh = true;
    },
    
    setNeedsRefresh: function(shouldRefresh)
    {
        this.needsRefresh = shouldRefresh;
    },
    
    needsRefresh: function()
    {
        return this.needsRefresh;
    },
        
    drawBackground: function(ctx)
    {
      var xStart = this.currentMinX;
      var numPoints = this.currentNumBuckets;
      
      //if ( Scale.current.key == SCALE_AUTO_KEY ) xStart -= 1;
      
      var fills = new Array("rgba(80,80,80, 0.15)","rgba(140,140,140, 0.25)");        
      var fill = 1;
      ctx.fillStyle = fills[fill];            

      if ( Scale.current.key == SCALE_1_WEEK_KEY || numPoints < 14)
      {
        // show days...
        var x = this.transformX( xStart );
        var iday = 0;
        while ( iday < numPoints+1 )
        {
           var xNext = this.transformX( xStart + iday + 1 );

           ctx.fillRect(x, 0, xNext - x, this.chartHeight);

           x = xNext;
           fill = 1 - fill;
           ctx.fillStyle = fills[fill];
           iday++;         
        }
      }
      else if ( Scale.current.key == SCALE_1_MONTH_KEY || 
                Scale.current.key == SCALE_MTD_KEY ||
                numPoints < 45)
      {
        // show weeks...
        // find first Sunday...
        var startDate = Utilities.dateFromDaysSinceEpoch(this.currentMinX+1);
        var firstUTCDate = new Date(startDate);
        var dayOfWeek = firstUTCDate.getDay();
        var wkRemainder = (dayOfWeek == 0 ) ? 0 : 7 - dayOfWeek;
        var iday = 0;
        var x = this.transformX( xStart + iday );
        
        while ( iday < numPoints+1 )
        {
           var xNext = this.transformX( xStart + iday + wkRemainder);

           ctx.fillRect(x, 0, xNext - x, this.chartHeight);

           x = xNext;
           fill = 1 - fill;
           ctx.fillStyle = fills[fill];
           iday += wkRemainder;
           wkRemainder = 7;
        }
      }
      else // show months...
      {
        //var todayXValue = Utilities.dateFromDaysSinceEpoch(Utilities.daysAgoUTC(0));
        var startDate = Utilities.dateFromDaysSinceEpoch(this.currentMinX+1);
        var todaysXValue = Utilities.numDaysSinceEpoch(Utilities.daysAgoUTC(0));
        var daysAgo = numPoints; // todayXValue - startDate;
        var firstUTCDate = new Date(startDate);
        var monthIndex = firstUTCDate.getMonth();
        var dayOfMonth = firstUTCDate.getDate();
        var daysInMonth = Utilities.daysInMonth(monthIndex, firstUTCDate);
        
        ctx.fillStyle = fills[fill];
        daysInMonth = (daysInMonth - dayOfMonth) + 2; 
        var iday = 0; // dayOfMonth;
        var x = this.transformX( xStart + iday);
        while ( iday < numPoints+1 )
        {
           var xNext = this.transformX( xStart + iday + daysInMonth - 1 );
           //var xWidth = Math.floor((this.width / numPoints)*daysInMonth);

           ctx.fillRect(x, 0, xNext - x, this.chartHeight);
           
           x = xNext;
           iday += daysInMonth;
           fill = 1 - fill;
           ctx.fillStyle = fills[fill];
           daysAgo -= daysInMonth;
           firstUTCDate = new Date(Utilities.daysAgoUTC(daysAgo - (this.currentMaxX - todaysXValue)));
           monthIndex = firstUTCDate.getMonth();
           dayOfMonth = firstUTCDate.getDate();
           daysInMonth = Utilities.daysInMonth(monthIndex, firstUTCDate);
        }
      }
    },
    
    determineCurrentMinMax: function()
    {
      if ( Scale.current.key == SCALE_AUTO_KEY )
      {
        Scale.current.adjustMinMax(this.dataSet[this.primaryIndex], this.primaryIndex);
        this.currentMinX = Scale.current.getXMin(this.primaryIndex);
        this.currentMaxX = Scale.current.getXMax(this.primaryIndex);
        
        var iC;
        for (iC = 0; iC < this.dataSet.length; iC++)
        {
           var config = this.chartConfig[iC];
           if ( !config || iC == this.primaryIndex || !this.chartConfig[iC].enabled ||
                !this.dataSet[iC].points || this.dataSet[iC].points.length == 0 ) continue;

           Scale.current.adjustMinMax(this.dataSet[iC], iC);

           var anXMin = Scale.current.getXMin(iC);
           var anXMax = Scale.current.getXMax(iC);
           
           if ( anXMin !=  SCALE_BIGNUM && anXMin < this.currentMinX ) 
           {
             this.currentMinX = anXMin;
           }
           if ( anXMax != -SCALE_BIGNUM && anXMax > this.currentMaxX )
           {
              this.currentMaxX = anXMax;
           }
        }
      }
      else
      {
        Scale.current.adjustMinMax(this.dataSet[this.primaryIndex], this.primaryIndex);
        this.currentMinX = Scale.current.getXMin(this.primaryIndex);
        this.currentMaxX = Scale.current.getXMax(this.primaryIndex);
      }
      
      this.currentNumBuckets = this.currentMaxX - this.currentMinX + 1;

      //  alert("BarChart.determineCurrentMinMax: # data pts: " + this.dataSet[this.primaryIndex].length + " num buckets: " + this.currentNumBuckets + " minX: " + this.currentMinX + " maxX: " + this.currentMaxX);
    },
    
    getReady: function()
    {
        this.determineCurrentMinMax();
        this.setWidth(Utilities.upperLimit(Utilities.lowerLimit(this.currentNumBuckets * 10, 200), 1600));
      
        var canvas = document.getElementById("barChartCanvas");
        var ctx = canvas.getContext("2d");
        //ctx.save();
        ctx.beginPath();
        ctx.clearRect(0,0, this.width, this.chartHeight + 1 );
    },
        
    draw: function()
    {
        if ( !this.needsRefresh ) return; 

        var canvas = document.getElementById("barChartCanvas");
        var ctx = canvas.getContext("2d");
        //ctx.restore();
        //ctx.save();
        ctx.beginPath();
        
        var iEn;
        var enabledIndices = new Array();
        enabledIndices[0] = this.primaryIndex;
        for (iEn = 0; iEn < this.dataSet.length; iEn++)
        {
           if ( iEn == this.primaryIndex ) continue;
           
           if ( this.chartConfig[iEn].enabled ) 
           {
             enabledIndices.push(iEn);
           }
        }
        
        this.determineCurrentMinMax();
        this.determineScale(this.primaryIndex);
        this.drawBackground(ctx);
                
        var ndx;
        var position = 0;
        for ( ndx = 0; ndx < enabledIndices.length; ndx++)
        {
            this.drawChart(ctx, enabledIndices[ndx], position, enabledIndices.length);
            position++;
        }
               
        this.constructOverlayDiv();
        this.setNeedsRefresh(false); 
        
        return;
    },
    
    drawChart: function(ctx, anIndex, position, total)  /* position: 0 = full width, 1 = far left, ...;  total: all datasets (including primary ) */
    {
      var redIncrPerPixel, greenIncrPerPixel, blueIncrPerPixel;
      
      Scale.current.adjustMinMax(this.dataSet[anIndex], anIndex);
      //this.determineCurrentMinMax();
      this.determineScale(anIndex);
      
      var numPoints = this.currentNumBuckets;
      var xPosArray = new Array();
      var yPosArray = new Array();
      var chartYMin = SCALE_BIGNUM;
      var chartYMax = -SCALE_BIGNUM;
            
      /*if ( this.chartConfig[anIndex].type == "area" ) ctx.beginPath();*/ 

      if ( numPoints > 0 && this.dataSet[anIndex] && this.dataSet[anIndex].length > 0)
      {
        if ( position == 0 )
        {
          redIncrPerPixel = (this.redEnd[anIndex] - this.redStart[anIndex]) / this.width;
          greenIncrPerPixel = (this.greenEnd[anIndex] - this.greenStart[anIndex]) / this.width;
          blueIncrPerPixel = (this.blueEnd[anIndex] - this.blueStart[anIndex]) / this.width;
        }
        
        ctx.lineWidth = 0.5;
        ctx.strokeStyle = "rgb(" + this.redStart[anIndex] + "," + this.greenStart[anIndex] + "," + this.blueStart[anIndex] + ")";
        var firstIndex = -1;
        var copyToIndex = 0;

        for ( ifx = 0; ifx < this.dataSet[anIndex].length; ifx++ )
        {
          if ( this.dataSet == null ||
               this.dataSet[anIndex][ifx] == null ||
               this.dataSet[anIndex][ifx].yValue == null ||
               this.dataSet[anIndex][ifx].xValue == null ||
               isNaN(this.dataSet[anIndex][ifx].yValue) ||
               isNaN(this.dataSet[anIndex][ifx].xValue) )
          {
              continue;
          }

          if ( this.dataSet[anIndex][ifx].xValue < this.currentMinX ||
               this.dataSet[anIndex][ifx].xValue > this.currentMaxX ) 
          {
              continue;
          }
            
          if ( firstIndex < 0 ) firstIndex = ifx;
          
          var xPos = this.transformX(this.dataSet[anIndex][ifx].xValue);
          var yPos = (this.chartConfig[anIndex].yMin != null && this.chartConfig[anIndex].yMin > this.dataSet[anIndex][ifx].yValue) ? this.chartHeight + 2 : this.transformY(this.dataSet[anIndex][ifx].yValue, anIndex);
          
          if ( this.dataSet[anIndex][ifx].yValue < chartYMin ) chartYMin = this.dataSet[anIndex][ifx].yValue;
          if ( this.dataSet[anIndex][ifx].yValue > chartYMax ) chartYMax = this.dataSet[anIndex][ifx].yValue;
          
          if ( this.chartConfig[anIndex].type == "area" )
          {
              xPos += (this.width / (2.0 * numPoints));
              ctx.fillStyle = (position == 0 ) ? "rgb(" + this.redEnd[anIndex] + "," + this.greenEnd[anIndex] +"," + this.blueEnd[anIndex] + ")" 
                                               : "rgba(" + this.redStart[anIndex] + "," + this.greenStart[anIndex] +"," + this.blueStart[anIndex] + ", 0.35 )";

              if ( ifx == firstIndex ) 
                ctx.moveTo(xPos, yPos);
              else
                ctx.lineTo(xPos, yPos);
                
              xPosArray[copyToIndex] = xPos;
              yPosArray[copyToIndex] = yPos;
              copyToIndex++;

          }
          else if ( this.chartConfig[anIndex].type == "line" )
          {
            xPos += (this.width / (2.0 * numPoints));              
            xPosArray[copyToIndex] = xPos;
            yPosArray[copyToIndex] = yPos;
            copyToIndex++;
          }
          else if ( this.chartConfig[anIndex].type == "bar" )
          {
            //alert("yValue: " + this.dataSet[anIndex][ifx].yValue + " y: " + yPos + " ifx: " + ifx);
            //alert("xValue: " + this.dataSet[anIndex][ifx].xValue + " x: " + xPos + " ifx: " + ifx);
            var fillStyleString;
            
            if ( position == 0 )
            {  
               fillStyleString = this.chartConfig[anIndex].yMin != null && this.chartConfig[anIndex].yMin > this.dataSet[anIndex][ifx].yValue ? "rgb(200,100,100)" :
                                     "rgb(" + Math.round(this.redStart[anIndex]   + xPos * redIncrPerPixel) + 
                                     "," + Math.round(this.greenStart[anIndex] + xPos * greenIncrPerPixel) + 
                                     "," + Math.round(this.blueStart[anIndex]  + xPos * blueIncrPerPixel) + ")";
               ctx.fillStyle = fillStyleString;
            }
            else
            {
                ctx.fillStyle =  "rgb(" + this.redEnd[anIndex] + "," + this.greenEnd[anIndex] +"," + this.blueEnd[anIndex] + ")";
            }
 
            var barWidth = (position == 0 ) ? this.width / numPoints : this.width / ( numPoints * (total + 1.0));
            var offset = (position == 0 ) ? 0 : barWidth * position;
            
            ctx.fillRect(xPos + offset, yPos, barWidth, this.chartHeight-yPos);
            ctx.strokeRect(xPos + offset, yPos, barWidth, this.chartHeight-yPos); 
            //if ( ifx < 2 )
            //      alert("xPos: "+ xPos + " yPos: " + yPos + " width: " + barWidth + " height: " + (this.chartHeight-yPos));
          }
        }
        
        if ( this.chartConfig[anIndex].type == "area" && xPosArray.length > 0 ) 
        {
          ctx.lineTo(xPos, this.chartHeight);
          ctx.lineTo(xPosArray[0], this.chartHeight);
          ctx.closePath();
          ctx.fill();
        }
          
        if ( (this.chartConfig[anIndex].type == "area" || this.chartConfig[anIndex].type == "line") && xPosArray.length > 0 ) 
        {
          // outline
          ctx.beginPath();
          ctx.lineWidth = 1.5;
          ctx.strokeStyle = "rgb(" + this.redStart[anIndex] + "," + this.greenStart[anIndex] + "," + this.blueStart[anIndex] + ")";
          for ( iPosArray = 0; iPosArray < xPosArray.length; iPosArray++)
          {
            xPos = xPosArray[iPosArray];
            yPos = yPosArray[iPosArray];
            
            if ( iPosArray == 0 ) 
              ctx.moveTo(xPos, yPos);
            else
              ctx.lineTo(xPos, yPos);              
          }
          
          if ( this.chartConfig[anIndex].type == "area" && xPosArray.length > 0 ) 
          {
            ctx.lineTo(xPos, this.chartHeight);
            ctx.lineTo(xPosArray[0], this.chartHeight);
            ctx.closePath();
          }
          
          ctx.stroke(); 

          // points
          //ctx.lineWidth = 1.0;
          ctx.fillStyle =  "rgb(" + this.redEnd[anIndex] + "," + this.greenEnd[anIndex] +"," + this.blueEnd[anIndex] + ")";
          for ( iPosArray = 0; iPosArray < xPosArray.length; iPosArray++)
          {
            xPos = xPosArray[iPosArray];
            yPos = yPosArray[iPosArray];
 
            var barWidth = (position == 0 ) ? this.width / numPoints : this.width / ( numPoints * (total + 2.0));
            
            ctx.fillRect(xPos-1, yPos-1, 3, 3);
//            ctx.strokeRect(xPos, yPos, barWidth, this.chartHeight-yPos); 
          }
        }
        
      }
      
      if ( position == 0 && this.chartConfig[anIndex].goal )
      {
        ctx.lineWidth = 1.5;
        ctx.strokeStyle = "rgb(" + this.redStart[anIndex] + "," + this.greenStart[anIndex] +"," + this.blueStart[anIndex] + ")";
        var dashLength = 10;
        var gapLength = 7;
        var xPos = 0; 
        var yPos = this.transformY(this.chartConfig[anIndex].goal, anIndex);
        ctx.beginPath();
        while ( xPos < this.width)
        {
           // alert("goal line: x: " + xPos + " y: " + yPos );
          ctx.moveTo(xPos, yPos);
          xPos += dashLength;
          ctx.lineTo(xPos, yPos);
          xPos += gapLength;
        }
        ctx.stroke();
      }

      // draw x-axis
      if ( anIndex == this.primaryIndex && chartYMin < 0 && chartYMax > 0 && (this.chartConfig[anIndex].yMin == null || this.chartConfig[anIndex].yMin < 0) )
      {
        ctx.lineWidth = 0.5;
        ctx.strokeStyle = "rgba(255,255,255,0.25)";
        var xPos = 0; 
        var yPos = this.transformY(0, anIndex);
        ctx.beginPath();
        ctx.moveTo(xPos, yPos);
        ctx.lineTo(this.width, yPos);
        ctx.stroke();
      }

    },
    
    constructOverlayDiv: function()
    {
      var noDataDiv = document.getElementById("noDataDiv");
      if ( (this.dataSet[this.primaryIndex] == null || this.dataSet[this.primaryIndex].length == 0) && noDataDiv != null )
      {
          noDataDiv.innerText = "No Data";
          this.overlayDOMDiv = noDataDiv;
          this.overlayDOMDiv.style.display = "block";
          return;
      }
      else if ( noDataDiv != null )
        noDataDiv.style.display = "none";
              
      this.determineScale(this.primaryIndex);
      
      this.overlayDOMDiv = document.createElement("div");

      var imageDiv = document.createElement("div");
      imageDiv.id = 'overlayImage';
      
      var listElement = document.createElement('ul');
      listElement.id = "dataSetList";
            
      var numPoints = this.currentNumBuckets;
      var pointsInRange = 0;

      for ( i = 0; i < this.dataSet[this.primaryIndex].length; i++)
      {
        if ( this.dataSet == null || 
             this.dataSet[this.primaryIndex][i] == null ||
             this.dataSet[this.primaryIndex][i].yValue == null ||
             this.dataSet[this.primaryIndex][i].xValue == null ||
             isNaN(this.dataSet[this.primaryIndex][i].yValue) ||
             isNaN(this.dataSet[this.primaryIndex][i].xValue) ) continue;

          if ( this.dataSet[this.primaryIndex][i].xValue < this.currentMinX ||
               this.dataSet[this.primaryIndex][i].xValue > this.currentMaxX ) continue;
               
          if ( this.chartConfig[this.primaryIndex].yMin != null && this.chartConfig[this.primaryIndex].yMin > this.dataSet[this.primaryIndex][i].yValue ) continue;
          
          pointsInRange++;
               
         var xPos = this.transformX(this.dataSet[this.primaryIndex][i].xValue);
         //alert("o-xValue: " + this.dataSet[this.primaryIndex][i].xValue + " x: " + xPos + " i: " + i);
         var yPos = this.chartConfig[this.primaryIndex].type == "bar" ? this.transformY(this.dataSet[this.primaryIndex][i].yValue, this.primaryIndex) : 2;
         
          //if ( i < 2 )
          //    alert("overlay: xPos: "+ xPos + " yPos: " + yPos + " width: " + ((this.width / numPoints) - 1) + " height: " + (this.height - parseInt(yPos)) - 20);

          // these list items define the hover area
         var dataSetListItem = document.createElement("li");
         dataSetListItem.id = i;
         dataSetListItem.style.left = parseInt(xPos) + "px";
         dataSetListItem.style.top = parseInt(yPos) + "px";
         dataSetListItem.style.width = ((this.width / numPoints) + 1) + "px";
         dataSetListItem.style.height = (this.height - parseInt(yPos)) - 20 + "px";
         /* dataSetListItem.addEventListener("mouseover", this.barHover, true); */
         
         var anchor = document.createElement("a");
         //anchor.href = "#";
         anchor.onclick = this.barSelected;
         anchor.onmouseover = this.barHover;
         anchor.onmouseout = this.barMouseOut;
         anchor.id = i;
       
         /* 
         <div id="overlayDOMDiv">
           <div id="overlayImage">
               <ul id="dataSetList">
                  <a><li><div><span></span></div></li></a>
                  <a><li><div><span></span></div></li></a>
                  <a><li><div><span></span></div></li></a>
                  <a><li><div><span></span></div></li></a>
               </ul>
            </div>
         </div>
         */
        
         
         anchor.appendChild(dataSetListItem);
         listElement.appendChild(anchor);
       }

       if ( pointsInRange == 0 && noDataDiv != null )
       {
           noDataDiv.innerText = "No Data In Range";
           this.overlayDOMDiv = noDataDiv;
           this.overlayDOMDiv.style.display = "block";
           return;
       }

       imageDiv.appendChild(listElement);
       this.overlayDOMDiv.appendChild(imageDiv);
    },

    formatYValueString: function(value)
    {
        if ( value == null || isNaN(value) ) return "";

        if ( this.units && this.units.indexOf("$") >= 0 )
             value = parseFloat(value).toFixed(2);
        else
             value = Utilities.toFixedStripped(parseFloat(value), 2);

        var isSingular = ( !isNaN(value) && value == 1 );
        var cvtdUnits = isSingular ? this.singularize(this.units) : this.units;
        
        return this.prefixUnits ? "" + cvtdUnits + value : "" + value + " " + cvtdUnits;
    },
    
    singularize: function(unitsString)
    {
        if ( !unitsString ) return "";
        
        var iesIndex = unitsString.lastIndexOf("ies");
        
        if ( iesIndex == unitsString.length - 3 )
            return unitsString.substring(0,iesIndex) + "y";
            
        var sIndex = unitsString.lastIndexOf("s");
        if ( sIndex == unitsString.length - 1 )
            return unitsString.substring(0,sIndex);
            
        return unitsString;
    },
        
    getOverlayDivContents: function() 
    {
      return this.overlayDOMDiv;
    },
    
    barSelected: function(anEvent)
    {
        if ( barClickCallback )
            barClickCallback(anEvent);
    },
            
    barHover: function(anEvent)
    {
        if ( barMouseOverCallback )
            barMouseOverCallback(anEvent);
    },
            
    barMouseOut: function(anEvent)
    {
        if ( barMouseOutCallback )
            barMouseOutCallback(anEvent);
    },
            
    transformX: function(xValue) 
    { 
        var result = Math.round((xValue - this.currentMinX) * this.xScale); 
        //alert("transformed: " + xValue + " to: " + result);
        return result;
    },

    transformY: function(yValue, anIndex) 
    { 
        var scale = Scale.current;
        var adjYMin = scale.getYMin(anIndex);

        if ( this.chartConfig[anIndex].yMin != null && this.chartConfig[anIndex].yMin < adjYMin ) adjYMin = this.chartConfig[anIndex].yMin;
        if ( this.chartConfig[anIndex].goal != null && this.chartConfig[anIndex].goal < adjYMin ) adjYMin = this.chartConfig[anIndex].goal;
         
        if ( adjYMin == scale.getYMax(anIndex) )    // happens when there is one point
        {
            return this.topBorderWidth ; // max the scale
        }
        
        // origin at upper left, hence the subtraction...
        result = this.chartHeight - (Math.round(( yValue - adjYMin ) * this.yScale[anIndex]) + this.bottomBorderWidth); 
        
        if ( isNaN(result) )
        {
            alert("transformY: result is not a number for: yValue: " + yValue + " adjYMin: " + adjYMin + " yScale: " + this.yScale[anIndex]);
        }
        return result;
    },
    
    determineScale: function(anIndex)
    {
        var scale = Scale.current;
        if ( !this.dataSet[anIndex] || this.dataSet[anIndex].length == 0) 
        {
          this.xScale = (this.width -  (this.leftBorderWidth + this.rightBorderWidth)) / this.currentNumBuckets; // pixels / range
          this.yScale[anIndex] = 1.0;
          return;
        }
        
        //scale.adjustMinMax(this.dataSet[anIndex], anIndex);
        
        var adjYMin = scale.getYMin(anIndex);
        if ( this.chartConfig[anIndex].yMin != null && this.chartConfig[anIndex].yMin < adjYMin ) adjYMin = this.chartConfig[anIndex].yMin;
        if ( this.chartConfig[anIndex].goal != null && this.chartConfig[anIndex].goal < adjYMin ) adjYMin = this.chartConfig[anIndex].goal;
 
        this.xScale = (this.width -  (this.leftBorderWidth + this.rightBorderWidth)) / this.currentNumBuckets; // pixels / range
        this.yScale[anIndex] = (scale.getYMax(anIndex) - adjYMin > 0.01) ? 
                                (this.chartHeight - (this.topBorderWidth + this.bottomBorderWidth)) / (scale.getYMax(anIndex) - adjYMin) :
                                1.0;
        //if ( this.xScale[0] != oldXScale ) alert("x scale changed from: " + oldXScale + " to: " + this.xScale[0] + " for xMin: " + scale.getXMin() + " xMax: " + scale.getXMax() );
        //alert("scale.getYMax(" + anIndex + "): " + scale.getYMax(anIndex) + " scale.getYMin(anIndex): " + adjYMin + " yScale[anIndex]: " + this.yScale[anIndex]);
        //alert("scale.getXMax(): " + scale.getXMax() + " scale.getXMin(): " + scale.getXMin() + " xScale[anIndex]: " + this.xScale);
        
        //if ( this.xScale > 50 )
        //    alert("width: " + this.width + " # buckets: " + this.currentNumBuckets);
    }

};
